@@ -6,9 +6,10 @@ Framework of Information Theory for Electrophysiological data and Statistics
 """
 import logging
 
-from frites import io, core, conn, stats, utils, workflow, simulations  # noqa
+from frites import (io, core, conn, stats, utils, workflow, simulations,  # noqa
+                    estimator)
 
-__version__ = "0.3.5"
+__version__ = "0.3.9"
 
 # -----------------------------------------------------------------------------
 # Set 'info' as the default logging level
