@@ -7,22 +7,53 @@ CONFIG = dict()
 
 # Supported MNE types
 MNE_EPOCHS_TYPE = (mne.Epochs, mne.EpochsArray, mne.epochs.EpochsFIF,
-                   mne.time_frequency.EpochsTFR, mne.epochs.BaseEpochs)
+                   mne.epochs.BaseEpochs)
 CONFIG["MNE_EPOCHS_TYPE"] = MNE_EPOCHS_TYPE
+CONFIG["MNE_EPOCHSTFR_TYPE"] = (mne.time_frequency.EpochsTFR)
 
 # Int and Float types
 INT_DTYPE = (int, np.int, np.int8, np.int16, np.int32, np.int64)
 FLOAT_DTYPE = (float, np.float, np.float16, np.float32, np.float64)
+STR_DTYPE = (str, np.str, np.string_)
 CONFIG['INT_DTYPE'] = INT_DTYPE
 CONFIG['FLOAT_DTYPE'] = FLOAT_DTYPE
+CONFIG['STR_DTYPE'] = STR_DTYPE
 
 # gcmi configuration
-CONFIG["KW_GCMI"] = dict(shape_checking=False, biascorrect=True, demeaned=True,
-                         mvaxis=-2, traxis=-1)
+CONFIG["KW_GCMI"] = dict(shape_checking=False, biascorrect=True,
+                         demeaned=False, mvaxis=-2, traxis=-1)
 
 # copula name conversion
 CONFIG["COPULA_CONV"] = dict(cc='gg', cd='gd', ccd='ggd')
 
+# mi types table
+CONFIG['MI_TABLE'] = {
+    'int': {
+        'none': 'cd',
+        'int': 'cd',
+        'float': 'none'
+    },
+    'float': {
+        'none': 'cc',
+        'int': 'ccd',
+        'float': 'ccc'
+    },
+    'none': {
+        'none': 'none',
+        'int': 'none',
+        'float': 'none',
+    }
+}
+
+# mi type full description
+CONFIG['MI_REPR'] = {
+    'none': 'none',
+    'cc': 'I(x; y (continuous))',
+    'cd': 'I(x; y (discret))',
+    'ccd': 'I(x; y (continuous)) | z (discret)',
+    'ccc': 'I(x; y (continuous)) | z (continuous)',
+}
+
 # general joblib config
 CONFIG["JOBLIB_CFG"] = dict()
 
@@ -57,3 +88,10 @@ CONFIG['MA_NAMES'] = [
     'R_PFrd', 'R_PFrm', 'R_OFCvl', 'R_OFCv', 'R_OFCvm', 'R_PFCvm', 'R_ACC',
     'R_Insula', 'L_Thal', 'L_Cd', 'L_Put', 'L_GP', 'L_Hipp', 'L_Amyg', 'L_NAc',
     'R_Thal', 'R_Cd', 'R_Put', 'R_GP', 'R_Hipp', 'R_Amyg', 'R_NAc']
+
+
+"""
+Default sigma for the hat correction when performing the t-test using
+MNE-Python
+"""
+CONFIG['TTEST_MNE_SIGMA'] = 0.001
