@@ -1,13 +1,15 @@
 """Workflow for computing MI and evaluate statistics."""
 import numpy as np
 import xarray as xr
-from joblib import Parallel, delayed
 
-from frites import config
+from mne.utils import ProgressBar
+
 from frites.io import set_log_level, logger
-from frites.core import get_core_mi_fun, permute_mi_vector
-from frites.workflow.wf_stats_ephy import WfStatsEphy
+from frites.core import permute_mi_vector
+from frites.workflow.wf_stats import WfStats
 from frites.workflow.wf_base import WfBase
+from frites.estimator import GCMIEstimator
+from frites.utils import parallel_func, kernel_smoothing
 
 
 class WfMi(WfBase):
@@ -37,16 +39,9 @@ class WfMi(WfBase):
               population.
 
         By default, the workflow uses group level inference ('rfx')
-    mi_method : {'gc', 'bin'}
-        Method for computing the mutual information. Use either :
-
-            * 'gc' : gaussian-copula based mutual information. This is the
-              fastest method but it can only captures monotonic relationships
-              between variables
-            * 'bin' : binning-based method that can captures any kind of
-              relationships but is much slower and also required to define the
-              number of bins to use. Note that if the Numba package is
-              installed computations should be much faster
+    estimator : MIEstimator | None
+        Estimator of mutual-information. If None, the Gaussian-Copula is used
+        instead.
     kernel : array_like | None
         Kernel for smoothing true and permuted MI. For example, use
         np.hanning(3) for a 3 time points smoothing or np.ones((3)) for a
@@ -57,7 +52,7 @@ class WfMi(WfBase):
     Friston et al., 1996, 1999 :cite:`friston1996detecting,friston1999many`
     """
 
-    def __init__(self, mi_type='cc', inference='rfx', mi_method='gc',
+    def __init__(self, mi_type='cc', inference='rfx', estimator=None,
                  kernel=None, verbose=None):
         """Init."""
         WfBase.__init__(self)
@@ -65,39 +60,27 @@ class WfMi(WfBase):
             "'mi_type' input parameter should either be 'cc', 'cd', 'ccd'")
         assert inference in ['ffx', 'rfx'], (
             "'inference' input parameter should either be 'ffx' or 'rfx'")
-        assert mi_method in ['gc', 'bin'], (
-            "'mi_method' input parameter should either be 'gc' or 'bin'")
         self._mi_type = mi_type
         self._inference = inference
-        self._mi_method = mi_method
-        self._need_copnorm = mi_method == 'gc'
-        self._gcrn = inference is 'rfx'
+        if estimator is None:
+            estimator = GCMIEstimator(mi_type=mi_type, copnorm=False,
+                                      verbose=verbose)
+        assert estimator.settings['mi_type'] == self._mi_type
+        self.estimator = estimator
+        self._copnorm = isinstance(estimator, GCMIEstimator)
+        self._gcrn = inference == 'rfx'
         self._kernel = kernel
         set_log_level(verbose)
         self.clean()
-        self._wf_stats = WfStatsEphy(verbose=verbose)
+        self._wf_stats = WfStats(verbose=verbose)
         # update internal config
-        self.update_cfg(mi_type=mi_type, inference=inference,
-            mi_method=mi_method, kernel=kernel)
+        self.attrs.update(dict(
+            mi_type=mi_type, inference=inference, kernel=kernel))
 
         logger.info(f"Workflow for computing mutual information ({inference} -"
-                    f" {mi_method} - {mi_type})")
+                    f" {mi_type})")
 
-    def _node_prepare_data(self, dataset):
-        """Prepare the data before computing the mi.
-
-        The preparation of the data depends both on the type of inferences
-        that are required and on the type of mutual information that is going
-        to be performed.
-        """
-        # inplace preparation
-        dataset.groupby("roi")
-        if self._need_copnorm:
-            dataset.copnorm(mi_type=self._mi_type, gcrn_per_suj=self._gcrn)
-        # track time and roi
-        self._times, self._roi = dataset.times, dataset.roi_names
-
-    def _node_compute_mi(self, dataset, n_bins, n_perm, n_jobs, random_state):
+    def _node_compute_mi(self, dataset, n_perm, n_jobs, random_state):
         """Compute mi and permuted mi.
 
         Permutations are performed by randomizing the regressor variable. For
@@ -105,52 +88,60 @@ class WfMi(WfBase):
         the random effect, the randomization is performed per subject.
         """
         # get the function for computing mi
-        mi_fun = get_core_mi_fun(self._mi_method)[self._mi_type]
-        assert f"mi_{self._mi_method}_ephy_{self._mi_type}" == mi_fun.__name__
+        mi_fun = self.estimator.get_function()
         # get x, y, z and subject names per roi
         if dataset._mi_type != self._mi_type:
             assert TypeError(f"Your dataset doesn't allow to compute the mi "
                              f"{self._mi_type}. Allowed mi is "
                              f"{dataset._mi_type}")
-        x, y, z, suj = dataset.x, dataset.y, dataset.z, dataset.suj_roi
-        n_roi, inf = dataset.n_roi, self._inference
+        # get data variables
+        n_roi = len(self._roi)
         # evaluate true mi
         logger.info(f"    Evaluate true and permuted mi (n_perm={n_perm}, "
                     f"n_jobs={n_jobs})")
-        mi = [mi_fun(x[k], y[k], z[k], suj[k], inf,
-                     n_bins=n_bins) for k in range(n_roi)]
-        # get joblib configuration
-        cfg_jobs = config.CONFIG["JOBLIB_CFG"]
+        # parallel function for computing permutations
+        parallel, p_fun = parallel_func(mi_fun, n_jobs=n_jobs, verbose=False)
+        pbar = ProgressBar(range(n_roi), mesg='Estimating MI')
         # evaluate permuted mi
-        mi_p = []
+        mi, mi_p = [], []
         for r in range(n_roi):
+            # get the data of selected roi
+            da = dataset.get_roi_data(
+                self._roi[r], copnorm=self._copnorm, mi_type=self._mi_type,
+                gcrn_per_suj=self._gcrn)
+            x, y, suj = da.data, da['y'].data, da['subject'].data
+            kw_mi = dict()
+            # cmi and categorical MI
+            if 'z' in list(da.coords):
+                kw_mi['z'] = da['z'].data
+            if self._inference == 'rfx':
+                kw_mi['categories'] = suj
+
+            # compute the true mi
+            _mi = mi_fun(x, y, **kw_mi)
             # get the randomize version of y
-            y_p = permute_mi_vector(y[r], suj[r], mi_type=self._mi_type,
-                                    inference=self._inference, n_perm=n_perm)
+            y_p = permute_mi_vector(
+                y, suj, mi_type=self._mi_type, inference=self._inference,
+                n_perm=n_perm)
             # run permutations using the randomize regressor
-            _mi = Parallel(n_jobs=n_jobs, **cfg_jobs)(delayed(mi_fun)(
-                x[r], y_p[p], z[r], suj[r], inf,
-                n_bins=n_bins) for p in range(n_perm))
-            mi_p += [np.asarray(_mi)]
-        # smoothing
-        if isinstance(self._kernel, np.ndarray):
-            logger.info("    Apply smoothing to the true and permuted MI")
-            for r in range(len(mi)):
-                for s in range(mi[r].shape[0]):
-                    mi[r][s, :] = np.convolve(
-                        mi[r][s, :], self._kernel, mode='same')
-                    for p in range(mi_p[r].shape[0]):
-                        mi_p[r][p, s, :] = np.convolve(
-                            mi_p[r][p, s, :], self._kernel, mode='same')
+            _mi_p = parallel(p_fun(x, y_p[p], **kw_mi) for p in range(n_perm))
+            _mi_p = np.asarray(_mi_p)
+
+            # kernel smoothing
+            if isinstance(self._kernel, np.ndarray):
+                _mi = kernel_smoothing(_mi, self._kernel, axis=-1)
+                _mi_p = kernel_smoothing(_mi_p, self._kernel, axis=-1)
+
+            mi += [_mi]
+            mi_p += [_mi_p]
+            pbar.update_with_increment_value(1)
 
         self._mi, self._mi_p = mi, mi_p
 
         return mi, mi_p
 
-
     def fit(self, dataset, mcp='cluster', n_perm=1000, cluster_th=None,
-            cluster_alpha=0.05, n_bins=None, n_jobs=-1, random_state=None,
-            **kw_stats):
+            cluster_alpha=0.05, n_jobs=-1, random_state=None, **kw_stats):
         """Run the workflow on a dataset.
 
         In order to run the worflow, you must first provide a dataset instance
@@ -192,11 +183,6 @@ class WfMi(WfBase):
         cluster_alpha : float | 0.05
             Control the percentile to use for forming the clusters. By default
             the 95th percentile of the permutations is used.
-        n_bins : int | None
-            Number of bins to use if the method for computing the mutual
-            information is based on binning (mi_method='bin'). If None, the
-            number of bins is going to be automatically inferred based on the
-            number of trials and variables
         n_jobs : int | -1
             Number of jobs to use for parallel computing (use -1 to use all
             jobs)
@@ -205,14 +191,15 @@ class WfMi(WfBase):
             If None, a random state is randomly assigned.
         kw_stats : dict | {}
             Additional arguments are sent to
-            :py:class:`frites.dataset.WfStatsEphy.fit`
+            :py:class:`frites.workflow.WfStats.fit`
 
         Returns
         -------
         mi, pvalues : array_like
-            DataArray of mutual information and p-values. If `inference` is
-            'ffx' the mi represents the MI computed across subjects while if it
-            is 'rfx' it's the mean across subjects.
+            DataArray of mutual information and p-values both of shapes
+            (n_times, n_roi). If `inference` is 'ffx' the mi represents the MI
+            computed across subjects while if it is 'rfx' it's the mean across
+            subjects.
 
         References
         ----------
@@ -224,11 +211,16 @@ class WfMi(WfBase):
         # don't compute permutations if mcp is either nostat / None
         if mcp in ['noperm', None]:
             n_perm = 0
-        # infer the number of bins if needed
-        if (self._mi_method is 'bin') and not isinstance(n_bins, int):
-            n_bins = 4
-            logger.info(f"    Use an automatic number of bins of {n_bins}")
-        self._n_bins = n_bins
+        # get needed dataset's informations
+        self._times, self._roi = dataset.times, dataset.roi_names
+        self._mi_dims = dataset._mi_dims
+        self._mi_coords = dict()
+        for k in self._mi_dims:
+            if k != 'roi':
+                self._mi_coords[k] = dataset.x[0].coords[k].data
+            else:
+                self._mi_coords['roi'] = self._roi
+        self._df_rs, self._n_subjects = dataset.df_rs, dataset._n_subjects
 
         # ---------------------------------------------------------------------
         # compute mutual information
@@ -240,75 +232,63 @@ class WfMi(WfBase):
                         "arguments")
             mi, mi_p = self._mi, self._mi_p
         else:
-            self._node_prepare_data(dataset)
             mi, mi_p = self._node_compute_mi(
-                dataset, self._n_bins, n_perm, n_jobs, random_state)
+                dataset, n_perm, n_jobs, random_state)
         """
         For information transfer (e.g FIT) we only need to compute the true and
         permuted mi but then, the statistics at the local representation mcp
         are discarded in favor of statistics on the information transfer
         """
-        if mcp is 'nostat':
+        if mcp == 'nostat':
             logger.debug("Permutations computed. Stop there")
             return None
 
-        # ---------------------------------------------------------------------
-        # 4d reshaping before the stats
-        # ---------------------------------------------------------------------
-        self._reshape = dataset._reshape
-        if self._reshape is not None:
-            logger.debug(f"    reshaping before computing statistics")
-            n_f, n_t = self._reshape
-            for k in range(len(mi)):
-                n_p, n_s, _ = mi_p[k].shape
-                mi[k] = mi[k].reshape(n_s, n_f, n_t)
-                mi_p[k] = mi_p[k].reshape(n_p, n_s, n_f, n_t)
-
         # ---------------------------------------------------------------------
         # compute statistics
         # ---------------------------------------------------------------------
+        # get additional stat arguments
+        kw_stats['tail'] = kw_stats.get('tail', 1)
         # infer p-values and t-values
         pvalues, tvalues = self._wf_stats.fit(
-            mi, mi_p, mcp=mcp, cluster_th=cluster_th, tail=1,
-            cluster_alpha=cluster_alpha, inference=self._inference,
-            **kw_stats)
-        # update internal config
-        self.update_cfg(n_perm=n_perm, random_state=random_state,
-                        n_bins=n_bins, **self._wf_stats.cfg)
+            mi, mi_p, cluster_th=cluster_th, inference=self._inference,
+            mcp=mcp, cluster_alpha=cluster_alpha, **kw_stats)
+        # update attributes
+        self.attrs.update(self._wf_stats.attrs)
+        self.attrs.update(dict(n_perm=n_perm, random_state=random_state))
 
         # ---------------------------------------------------------------------
         # postprocessing and conversions
         # ---------------------------------------------------------------------
         # tvalues conversion
         if isinstance(tvalues, np.ndarray):
-            self._tvalues = self._xr_conversion(tvalues)
+            self._tvalues = self._xr_conversion(tvalues, 'tvalues')
         # mean mi across subjects
-        if self._inference is 'rfx':
+        if self._inference == 'rfx':
             logger.info("    Mean mi across subjects")
             mi = [k.mean(axis=0, keepdims=True) for k in mi]
         mi = np.moveaxis(np.concatenate(mi, axis=0), 0, -1)
         # dataarray conversion
-        mi = self._xr_conversion(mi)
-        pv = self._xr_conversion(pvalues)
+        mi = self._xr_conversion(mi, 'mi')
+        pv = self._xr_conversion(pvalues, 'pvalues')
 
         return mi, pv
 
-    def _xr_conversion(self, x):
+    def _xr_conversion(self, x, name):
         """Xarray conversion."""
-        times, roi = self._times, self._roi
-        if x.ndim == 2:
-            x_da = xr.DataArray(x, dims=('times', 'roi'), coords=(times, roi))
-        elif x.ndim == 3:
-            freqs = np.arange(x.shape[0])
-            x_da = xr.DataArray(x, dims=('freqs', 'times', 'roi'),
-                                coords=(freqs, times, roi))
-        self._attrs_xarray(x_da)
+        # build dimension order
+        dims = ['times', 'roi']
+        supp_dim = [k for k in self._mi_dims if k not in dims]
+        dims = supp_dim + dims
+        # build coordinates
+        coords = [self._mi_coords[k] for k in dims]
+        # build xarray
+        da = xr.DataArray(x, dims=dims, coords=coords)
+        # wrap with workflow's attributes
+        da = self.attrs.wrap_xr(da, name=name)
+        return da
 
-        return x_da
-
-
-    def conjunction_analysis(self, dataset, p=.05, mcp='cluster',
-                             cluster_th=None, cluster_alpha=0.05):
+    def conjunction_analysis(self, p=.05, mcp='cluster', cluster_th=None,
+                             cluster_alpha=0.05):
         """Perform a conjunction analysis.
 
         This method can be used in order to determine the number of subjects
@@ -325,9 +305,6 @@ class WfMi(WfBase):
 
         Parameters
         ----------
-        dataset : :class:`frites.dataset.DatasetEphy`
-            A dataset instance. Note that it should be the same dataset used
-            with the fit() method.
         p : float | 0.05
             Significiency threshold to find significant effect per subject.
         kwargs : dict | {}
@@ -352,29 +329,30 @@ class WfMi(WfBase):
             "perform the conjunction analysis.")
 
         # retrieve the original number of subjects
-        n_roi = len(self._mi)
         pv_s = {}
-        for s in range(dataset.n_subjects):
+        for s in range(self._n_subjects):
             # reconstruct the mi and mi_p of each subject
             mi_s, mi_ps, roi_s = [], [], []
-            for r in range(n_roi):
-                suj_roi_u = dataset.suj_roi_u[r]
+            for n_r, r in enumerate(self._roi):
+                suj_roi_u = np.array(self._df_rs.loc[r, 'subjects'])
                 if s not in suj_roi_u: continue  # noqa
                 is_suj = suj_roi_u == s
-                mi_s += [self._mi[r][is_suj, :]]
-                mi_ps += [self._mi_p[r][:, is_suj, :]]
-                roi_s += [self._roi[r]]
+                mi_s += [self._mi[n_r][is_suj, :]]
+                mi_ps += [self._mi_p[n_r][:, is_suj, :]]
+                roi_s += [self._roi[n_r]]
 
             # perform the statistics
             _pv_s = self._wf_stats.fit(
-            mi_s, mi_ps, mcp=mcp, cluster_th=cluster_th, tail=1,
-            cluster_alpha=cluster_alpha, inference='ffx')[0]
+                mi_s, mi_ps, mcp=mcp, cluster_th=cluster_th, tail=1,
+                cluster_alpha=cluster_alpha, inference='ffx')[0]
             # dataarray conversion
             pv_s[s] = xr.DataArray(_pv_s < p, dims=('times', 'roi'),
                                    coords=(self._times, roi_s))
         # cross-subjects conjunction
         conj_ss = xr.Dataset(pv_s).to_array('subject')
+        conj_ss.name = 'Single subject conjunction'
         conj = conj_ss.sum('subject')
+        conj.name = 'Across subjects conjunction'
         # add attributes to the dataarray
         attrs = dict(p=p, cluster_th=cluster_th, cluster_alpha=cluster_alpha,
                      mcp=mcp)
@@ -384,6 +362,70 @@ class WfMi(WfBase):
 
         return conj_ss, conj
 
+    def get_params(self, *params):
+        """Get formatted parameters.
+
+        This method can be used to get internal arrays formatted as xarray
+        DataArray.
+
+        Parameters
+        ----------
+        params : string
+            Internal array names to get as xarray DataArray. You can use :
+
+                * 'tvalues' : DataArray of t-values of shape (n_times, n_roi).
+                  Only possible with RFX inferences
+                * 'mi_ss' : DataArray of single subject mutual-information of
+                  shape (n_subjects, n_times, n_roi)
+                * 'perm_ss' : DataArray of computed permutations of shape
+                  (n_perm, n_subjects, n_times, n_roi)
+                * 'perm_' : DataArray of maximum computed permutations of
+                  shape (n_perm,)
+        """
+        # get coordinates
+        times, roi, df_rs = self._times, self._roi, self._df_rs
+        if self._inference == 'ffx':
+            suj = [np.array([-1])] * len(roi)
+        elif self._inference == 'rfx':
+            suj = [np.array(df_rs.loc[r, 'subjects']) for r in roi]
+        n_perm = self._mi_p[0].shape[0]
+        perm = np.arange(n_perm)
+        # loop over possible outputs
+        outs = []
+        for param in params:
+            assert isinstance(param, str)
+            logger.info(f'    Formatting array {param}')
+            if param == 'tvalues':
+                da = self._tvalues
+            elif param == 'mi_ss':
+                mi = dict()
+                for n_r, r in enumerate(roi):
+                    mi[r] = xr.DataArray(
+                        self._mi[n_r], coords=(suj[n_r], times),
+                        dims=('subject', 'times'))
+                da = xr.Dataset(mi).to_array('roi')
+                da = da.transpose('subject', 'times', 'roi')
+            elif param == 'perm_ss':
+                mi = dict()
+                for n_r, r in enumerate(roi):
+                    mi[r] = xr.DataArray(
+                        self._mi_p[n_r], dims=('perm', 'subject', 'times'),
+                        coords=(perm, suj[n_r], times))
+                da = xr.Dataset(mi).to_array('roi')
+                da = da.transpose('perm', 'subject', 'times', 'roi')
+            elif param == 'perm_':
+                mi_p = np.r_[tuple([k.ravel() for k in self._mi_p])]
+                mi_p.sort()
+                da = xr.DataArray(mi_p[-n_perm:], dims=('perm',),
+                                  coords=(perm,))
+            else:
+                raise ValueError(f"Parameter {param} not found")
+            # add workflow attributes
+            self.attrs.wrap_xr(da, name=param)
+            outs += [da]
+
+        return tuple(outs)
+
     def clean(self):
         """Clean computations."""
         self._mi, self._mi_p, self._tvalues = [], [], None
